local msq, msqTools
msq = LibStub("Masque",true)
if msq then
	msqTools = msq:Group("Farmhand")
end

local SZ = LibStub("LibBabble-SubZone-3.0"):GetLookupTable()

local _
local ButtonSpacing = 6

local Farmhand = CreateFrame("frame","Farmhand",UIParent)
Farmhand:RegisterEvent("PLAYER_LOGIN")
Farmhand:RegisterEvent("PLAYER_ENTERING_WORLD")
local function Farmhand_EventHandler(self, event, ...)
	if event == "PLAYER_LOGIN" then
		self:Initialize()
	elseif event == "PLAYER_ENTERING_WORLD" then
		Farmhand:RegisterEvent("BAG_UPDATE")
		Farmhand:RegisterEvent("ZONE_CHANGED")
		Farmhand:UnregisterEvent("PLAYER_ENTERING_WORLD")
	elseif event == "ZONE_CHANGED" then
		Farmhand:UpdateLayout()
	elseif event == "BAG_UPDATE" then
		Farmhand:UpdateLayout()
	elseif event == "PLAYER_REGEN_ENABLED" then
		Farmhand:UnregisterEvent("PLAYER_REGEN_ENABLED")
		if self.Initialized then
			self:UpdateLayout()
		else
			self:Initialize()
		end	
	end
end
Farmhand:SetScript("OnEvent",Farmhand_EventHandler)
Farmhand:SetMovable(true)

Farmhand.tools = CreateFrame("Frame","FarmhandTools",Farmhand);
Farmhand.tools:SetScale(1.5)
Farmhand.tools.items = {
	{id=89815}, --Master Plow
	{id=89880}, --Dented Shovel
	{id=80513}, --Vintage Bug Sprayer
	{id=79104}, --Rusty Watering Can
}

Farmhand.lock = CreateFrame("CheckButton","FarmhandToolLock",FarmhandTools);
Farmhand.lock:SetBackdrop({bgFile="Interface\\Buttons\\LockButton-Border"});
Farmhand.lock:SetNormalTexture("Interface\\Buttons\\LockButton-Unlocked-Up");
Farmhand.lock:SetPushedTexture("Interface\\Buttons\\LockButton-Unlocked-Down");
Farmhand.lock:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
Farmhand.lock:SetCheckedTexture("Interface\\Buttons\\LockButton-Locked-Up");
Farmhand.lock:SetSize(16,16)
Farmhand.lock:SetPoint("Right",FarmhandTools,"Left",0,0)

RegisterStateDriver(Farmhand.lock,"visibility","[mod:shift] show; hide")

Farmhand.lock:SetScript("OnClick",function()
	FarmhandData.ToolsLocked = Farmhand.lock:GetChecked() == 1
end)

Farmhand.seeds = CreateFrame("Frame","FarmhandSeeds",Farmhand);
Farmhand.seeds:SetScale(1)
Farmhand.seeds.items = {
	{id=79102}, --Green Cabbage Seeds
	{id=89328}, --Jade Squash Seeds
	{id=80590}, --Juicycrunch Carrot Seeds
	{id=80592}, --Mogu Pumpkin Seeds
	{id=80594}, --Pink Turnip Seeds
	{id=80593}, --Red Blossom Leek Seeds
	{id=80591}, --Scallion Seeds
	{id=89329}, --Striped Melon Seeds
	{id=80595}, --White Turnip Seeds
	{id=89326}, --Witchberry Seeds
	{id=85219}, --Ominous Seed
	{id=85216}, --Enigma Seed
	{id=85217}, --Magebulb Seed
	{id=89202}, --Raptorleaf Seed
	{id=85215}, --Snakeroot Seed
	{id=89233}, --Songbell Seed
	{id=89197}, --Windshear Cactus Seed
}

function Farmhand:Initialize()
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	if FarmhandData == nil then
		FarmhandData = {
			X = 0,
			Y = -250,
			ToolsLocked = false,
		}
	end
	Farmhand.InHalfHill = false
	Farmhand:Hide()

	Farmhand.lock:SetChecked(FarmhandData.ToolsLocked == true)
	
	self:CreateButtons(Farmhand.tools)
	self:CreateButtons(Farmhand.seeds)
	self:UpdateLayout()
	self.Initialized = true
end

function Farmhand:CreateButtons(bar)
	for i,item in ipairs(bar.items) do

		local icon = select(10,GetItemInfo(item.id))

		item.btn = CreateFrame("Button", bar:GetName().."Button"..i, bar, "SecureActionButtonTemplate");
		item.btn:SetSize(32,32)

		item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
		item.btn.icon:SetTexture(icon)
		item.btn.icon:SetAllPoints()
		item.btn.icon:Show()

		item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
		item.btn.count:SetJustifyH("RIGHT")
		item.btn.count:SetPoint("BottomRight",-2,2)

		item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
		item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
		
		item.btn:RegisterForClicks("AnyUp")
		item.btn:SetAttribute("type", "item");
		item.btn:SetAttribute("item1", "item:"..item.id);
		item.btn:SetScript("OnMouseDown",Farmhand.OnMouseDown)
		item.btn:SetScript("OnMouseUp",Farmhand.OnMouseUp)
		item.btn:SetScript("OnHide",Farmhand.OnHide)
		item.btn:SetScript("OnEnter",Farmhand.OnEnter)
		item.btn:SetScript("OnLeave",function() GameTooltip_Hide() end)
		if msqTools then
			msqTools:AddButton(item.btn)
		end
	end
end

function Farmhand:OnEnter()
	local _, link = GetItemInfo(self:GetAttribute("item1"))
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetHyperlink(link)
	GameTooltip:Show()
end

function Farmhand:OnMouseDown(button)
	if IsShiftKeyDown() and button == "LeftButton" and not Farmhand.isMoving then
		Farmhand:StartMoving()
		Farmhand.isMoving = true
	elseif IsShiftKeyDown() and button == "RightButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing();
		FarmhandData.X, FarmhandData.Y = 0, -250
		Farmhand:SetPoint("Center",UIParent,"Center",0,-250)
		Farmhand.isMoving = false;
	end
end

function Farmhand:OnMouseUp(button)
	if button == "LeftButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		_,_,_,FarmhandData.X,FarmhandData.Y = Farmhand:GetPoint(1)
		Farmhand.isMoving = false
   end
end

function Farmhand:OnHide()
	if ( Farmhand.isMoving ) then
		Farmhand:StopMovingOrSizing();
		_,_,_,FarmhandData.X,FarmhandData.Y = Farmhand:GetPoint(1)
		Farmhand.isMoving = false;
	end
end

function Farmhand:UpdateBar(bar)
	local last
	local shown = 0
	for _,item in ipairs(bar.items) do
		local ItemCount = GetItemCount(item.id)
		if ItemCount > 0 then
			if shown % 8 == 0 then
				local row = math.floor(shown/8)
				item.btn:SetPoint("TopLeft",bar,"TopLeft", ButtonSpacing/2, -(ButtonSpacing * .5 ) - row*item.btn:GetHeight() - row*ButtonSpacing)
				last = item.btn
			else
				item.btn:SetPoint("TopLeft",last,"TopRight",ButtonSpacing,0)
				last = item.btn
			end
			if IsConsumableItem(item.id) then
				local count = _G[item.btn:GetName().."Count"]
				if ItemCount > 999 then
					count:SetText("*")
				else
					count:SetText(ItemCount)
				end
			end
			local bag, slot = self:FindItemInBags(item.id)
			if bag and slot then
				item.btn:SetAttribute("item2",bag.." "..slot)
			end
			item.btn:Show()
			shown = shown + 1
		else
			item.btn:Hide()
		end
	end
	
	if last ~= nil then
		bar:SetWidth(math.min(8,shown) * (last:GetWidth() + ButtonSpacing))
		bar:SetHeight(math.ceil(shown/8) * (last:GetHeight() + ButtonSpacing))
		bar:Show()
	else
		bar:Hide()
	end
end

function Farmhand:UpdateLayout()

	if GetZoneText() == "" then return end

	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	if GetSubZoneText() ~= SZ["Sunsong Ranch"] and  
	   GetSubZoneText() ~= SZ["The Halfhill Market"] then

		Farmhand:Hide()
		if GetSubZoneText() ~= SZ["Halfhill"] and Farmhand.InHalfhill then
			Farmhand.InHalfhill = false
			if FarmhandData.ToolsLocked == true then
				print("Leaving "..SZ["Halfhill"]..". Tools are Locked.")
			else
				ClearCursor()
				for i,item in ipairs(Farmhand.tools.items) do
					local bag, slot = self:FindItemInBags(item.id)
					if bag and slot then
						PickupContainerItem(bag,slot)
						if CursorHasItem() then
							local _, id, link = GetCursorInfo()
							if id == item.id then
								print("Leaving "..SZ["Halfhill"]..". Dropping "..link..".")
								DeleteCursorItem()
							end
						end
					end
				end
			end
		end
		return
		
	end

	Farmhand.InHalfhill = true
	
	Farmhand:UpdateBar(self.tools)
	Farmhand:UpdateBar(self.seeds)
		
	local th = (Farmhand.tools:IsShown() and Farmhand.tools:GetHeight()*Farmhand.tools:GetScale() or 0)
	local sh = (Farmhand.seeds:IsShown() and Farmhand.seeds:GetHeight()*Farmhand.seeds:GetScale() or 0)
	Farmhand:SetHeight(24 + th + sh)
	
	local tw = (Farmhand.tools:IsShown() and Farmhand.tools:GetWidth()*Farmhand.tools:GetScale() or 0)
	local sw = (Farmhand.seeds:IsShown() and Farmhand.seeds:GetWidth()*Farmhand.seeds:GetScale() or 0)
	Farmhand:SetWidth(24 + math.max(tw,sw))
	
	if Farmhand.tools:IsShown() or Farmhand.seeds:IsShown() then
		Farmhand:Show()
	else
		Farmhand:Hide()
	end

	Farmhand.seeds:SetPoint("Top",Farmhand,"Top",0,-15)
	Farmhand.tools:SetPoint("Bottom",Farmhand,"Bottom",0,7)
	Farmhand.tools:SetPoint("Bottom",Farmhand,"Bottom",0,7)
	Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
end

function Farmhand:FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 0, numSlots do
				for slot = 1, numSlots do
					if item == GetContainerItemID(container, slot) then
						return container, slot
					end
				end
			end
		end
	end
end
